;
;   c n _ d a t e / c v _ i l d a t 
;   c n _ d a y   / c v _ i l d a y
;
;   Does the QDOS-vector for C, to convert a QDOS-format date 
;   (local time in seconds from 1961 Jan 01 00:00:00)
;   into:
;       - date and time                 cn_date / cv_ildat
;       - day of the week               cn_day  / cv_ilday
;
;   Value returned us a '\0'-terminated character vector (C-string).
;
;   Assumptions:
;       - Function return value passed in d0
;       - Compiler must _NOT_ assume that a0, a1, d1 are preserved
;         in function calls!
;
;   Equivalent to C routine:
;       char *cn_date (char * buffer, long qdos_date)
;       char *cn_day  (char * buffer, long_qdos_date)
;
;       buffer must have at least 25 characters for cn_date, 
;                    and at least 7  characters for cn_day
;
;   cn_day returns a pointer to '\0'-terminated char vector.
;   WARNING: the result of the function is __N_O_T__ the start of the buffer!!
;         The result is buffer+2, the first word (Motorola format, 2 bytes)
;         holds the length of the string
;
;   AMENDMENT HISTORY
;   ~~~~~~~~~~~~~~~~
;               PH    - Original version by Peter Holzer
;
;   11 Mar 93   DJW   - Added cn_day entry point, and slightly reworked
;
;   29 Jul 93   DJW   - Addedthe SMS entry points
;
;   06 Nov 93   DJW   - Added underscore to entry point names
;
;   24 Jan 94   DJW   - Added (yet another) underscore to entry point names
;                       for name hiding purposes

    .text
    .even

    .globl  __cn_date     ; QDOS name for call
    .globl  __cv_ildat    ; SMS name for call
__cn_date:
__cv_ildat:
    move.w  $ec,a0      ; vector CN.DATE
    moveq   #22,d0      ; length to last character fo string returned
    bra     share

    .globl  __cn_day      ; QDOS name for call
    .globl  __cv_day      ; SMS name for call
__cn_day:
__cv_day:
    move.w  $ee,a0          ; vector CN.DAY
    moveq   #6,d0           ; length to last character fo string returned


SAVEREG equ 4+4         ; size of saved registers + return address

share:
    move.l a6,-(a7)         ; Uses a6 as a base register (a1 is relative to a6)
    move.l 0+SAVEREG(a7),a1 ; get first argument, i.e. pointer to buffer (ascdat)
    move.l 4+SAVEREG(a7),d1 ; get second argument (qdat)
    adda.l d0,a1            ; point to last char of string which will be returned
    clr.b  (a1)             ; make the terminating '\0';
    suba.l a6,a6            ; this makes a1 to an absolute pointer
    jsr    (a0)             ; do conversion
    move.l a1,d0            ; load return register with pointer to QDOS-string
    addq.l #2,d0            ; point to start of C-string
    movea.l (a7)+,a6        ; restore a6
    rts

