;
;       _ s u p e r   /   _ u s e r
;
; Routines for entering/leaving Supervisor mode
;
; this is very C specific, the local variables must be addressed via a6 or
; another address register. the routine must be called with no parameters
; on the stack, or it will mess up on return. the routine _user must be called
; before exiting the calling routine.
;
; Equivalent to C routines:
;
;   void _super (void)
;   void _user  (void)
;
; AMENDMENT HISTORY
; ~~~~~~~~~~~~~~~~~
;
;   06 Nov 93   DJW   - Added underscore to entry point names
;
;   13 Apr 94   EAJ   - _super() saves SR on stack, and _user() returns to
;                       the saved mode. This allows for nested calls to
;                       _super/_user(). Only the last call to _user() will
;                       return to user mode. Why this ?
;                       Example: Someone writes a function called foo(), which
;                       contains a call to _super()/_user(). Someone else,
;                       who does not know the internals of foo(), should be
;                       able to write a function baz(), which also calls
;                       _super()/_user(), with a call to foo() inside.
;
;   06 Sep 95   RZ    - added check for pending signals after returning
;                       to user mode
;

    .text
    .even
    .globl __super
    .globl __user

__super:                ; go to supervisor mode - only change stack pointer
    move.l   (a7)+,d0   ; get old return address from user stack
    move.w   sr,d1      ; get sr (including current mode)
    trap     #0         ; go to supervisor mode
    move.w   d1,-(a7)   ; save old sr on new stack
    move.l   d0,-(a7)   ; set supervisor stack up for the return
    rts

__user:                 ; return to previous mode (!)
    move.l   (a7)+,d0   ; get return address from supervisor stack
    move.w   (a7)+,d1   ; get saved sr
    move.w   d1,sr      ; restore saved mode
    move.l   d0,-(a7)   ; set return address on stack
    andi.w   #$2000,d1  ; user mode again ?
    bne      urt
    jsr      __CheckSig ; ..  check and raise pending signals
urt:
    rts

